ConstantExprKind

Enum ConstantExprKind 

pub enum ConstantExprKind {
Show 25 variants Bool(bool), Integer(IntegerValue), Char(char), Float(FloatValue), Adt(Option<VariantId>, Vec<ConstantExpr>), Array(Vec<ConstantExpr>), Ref(ConstantExpr, Option<UnsizingMetadata>), Ptr(RefKind, ConstantExpr, Option<UnsizingMetadata>), Str(String), ByteStr(Vec<u8>), FnDef(FnPtr), FnPtr(FnPtr), PtrNoProvenance(u128), TypeId(Ty), RawMemory(Vec<Byte>), Var(ConstGenericDbVar), Global(GlobalDeclRef), Call(FnPtr, Vec<ConstantExpr>), TraitConst(TraitRef, AssocConstId), VTableRef(TraitRef), Discriminant(TypeDeclRef, VariantId), SizeOf(Ty), AlignOf(Ty), OffsetOf(TypeDeclRef, Option<VariantId>, FieldId), Opaque(String),
}

Variants



Bool(bool)

Boolean value.



Integer(IntegerValue)

Integer value.



Char(char)

Char value.



Float(FloatValue)

Float value.



Adt(Option<VariantId>, Vec<ConstantExpr>)

Value of an ADT (struct or enum).

This is eliminated inside functions if --raw-consts is off.



Array(Vec<ConstantExpr>)

Array value.

This is eliminated inside functions if --raw-consts is off.



Ref(ConstantExpr, Option<UnsizingMetadata>)

A shared reference to a constant value.

This is eliminated inside functions if --raw-consts is off.



Ptr(RefKind, ConstantExpr, Option<UnsizingMetadata>)

A pointer to a static.

This is eliminated inside functions if --raw-consts is off.



Str(String)

str value.



ByteStr(Vec<u8>)

Byte string value.



FnDef(FnPtr)

ZST constant corresponding to the unique value of the type of a function item.



FnPtr(FnPtr)

A function pointer value; this is a pointer (i.e. an address).

This is eliminated inside functions if --raw-consts is off.



PtrNoProvenance(u128)

A pointer with no provenance (e.g. 0 for the null pointer)

This is eliminated inside functions if --raw-consts is off.



TypeId(Ty)

The TypeId value for a type.



RawMemory(Vec<Byte>)

Raw memory value obtained from constant evaluation. Used when a more structured representation isn’t possible (e.g. for unions) or just isn’t implemented yet.



Var(ConstGenericDbVar)

A const generic var



Global(GlobalDeclRef)

The value of a constant/static.

This is eliminated inside functions if --raw-consts is off.



Call(FnPtr, Vec<ConstantExpr>)

A call to a const fn.



TraitConst(TraitRef, AssocConstId)

A trait associated constant.

E.g.:

impl Foo for Bar {
  const C : usize = 32; // <-
}


VTableRef(TraitRef)

A reference to the vtable static item for this trait ref. This can be normalized if we emitted a vtable item.

This is eliminated if --raw-consts is off.



Discriminant(TypeDeclRef, VariantId)

The integer discriminant value corresponding to this enum variant.



SizeOf(Ty)

The size of the given type.



AlignOf(Ty)

The alignment of the given type.



OffsetOf(TypeDeclRef, Option<VariantId>, FieldId)

The offset of the given field.



Opaque(String)

A constant expression that Charon doesn’t handle, along with the reason why.

Implementations

Source

impl ConstantExprKind

pub fn variant_name(&self) -> &'static str

Source

impl ConstantExprKind

pub fn is_bool(&self) -> bool

pub fn is_integer(&self) -> bool

pub fn is_char(&self) -> bool

pub fn is_float(&self) -> bool

pub fn is_adt(&self) -> bool

pub fn is_array(&self) -> bool

pub fn is_ref(&self) -> bool

pub fn is_ptr(&self) -> bool

pub fn is_str(&self) -> bool

pub fn is_byte_str(&self) -> bool

pub fn is_fn_def(&self) -> bool

pub fn is_fn_ptr(&self) -> bool

pub fn is_ptr_no_provenance(&self) -> bool

pub fn is_type_id(&self) -> bool

pub fn is_raw_memory(&self) -> bool

pub fn is_var(&self) -> bool

pub fn is_global(&self) -> bool

pub fn is_call(&self) -> bool

pub fn is_trait_const(&self) -> bool

pub fn is_vtable_ref(&self) -> bool

pub fn is_discriminant(&self) -> bool

pub fn is_size_of(&self) -> bool

pub fn is_align_of(&self) -> bool

pub fn is_offset_of(&self) -> bool

pub fn is_opaque(&self) -> bool

Source

impl ConstantExprKind

pub fn as_bool(&self) -> Option<&bool>

pub fn as_integer(&self) -> Option<&IntegerValue>

pub fn as_char(&self) -> Option<&char>

pub fn as_float(&self) -> Option<&FloatValue>

pub fn as_adt(&self) -> Option<(&Option<VariantId>, &Vec<ConstantExpr>)>

pub fn as_array(&self) -> Option<&Vec<ConstantExpr>>

pub fn as_ref(&self) -> Option<(&ConstantExpr, &Option<UnsizingMetadata>)>

pub fn as_ptr( &self, ) -> Option<(&RefKind, &ConstantExpr, &Option<UnsizingMetadata>)>

pub fn as_str(&self) -> Option<&String>

pub fn as_byte_str(&self) -> Option<&Vec<u8>>

pub fn as_fn_def(&self) -> Option<&FnPtr>

pub fn as_fn_ptr(&self) -> Option<&FnPtr>

pub fn as_ptr_no_provenance(&self) -> Option<&u128>

pub fn as_type_id(&self) -> Option<&Ty>

pub fn as_raw_memory(&self) -> Option<&Vec<Byte>>

pub fn as_var(&self) -> Option<&ConstGenericDbVar>

pub fn as_global(&self) -> Option<&GlobalDeclRef>

pub fn as_call(&self) -> Option<(&FnPtr, &Vec<ConstantExpr>)>

pub fn as_trait_const(&self) -> Option<(&TraitRef, &AssocConstId)>

pub fn as_vtable_ref(&self) -> Option<&TraitRef>

pub fn as_discriminant(&self) -> Option<(&TypeDeclRef, &VariantId)>

pub fn as_size_of(&self) -> Option<&Ty>

pub fn as_align_of(&self) -> Option<&Ty>

pub fn as_offset_of( &self, ) -> Option<(&TypeDeclRef, &Option<VariantId>, &FieldId)>

pub fn as_opaque(&self) -> Option<&String>

Source

impl ConstantExprKind

pub fn as_bool_mut(&mut self) -> Option<&mut bool>

pub fn as_integer_mut(&mut self) -> Option<&mut IntegerValue>

pub fn as_char_mut(&mut self) -> Option<&mut char>

pub fn as_float_mut(&mut self) -> Option<&mut FloatValue>

pub fn as_adt_mut( &mut self, ) -> Option<(&mut Option<VariantId>, &mut Vec<ConstantExpr>)>

pub fn as_array_mut(&mut self) -> Option<&mut Vec<ConstantExpr>>

pub fn as_ref_mut( &mut self, ) -> Option<(&mut ConstantExpr, &mut Option<UnsizingMetadata>)>

pub fn as_ptr_mut( &mut self, ) -> Option<(&mut RefKind, &mut ConstantExpr, &mut Option<UnsizingMetadata>)>

pub fn as_str_mut(&mut self) -> Option<&mut String>

pub fn as_byte_str_mut(&mut self) -> Option<&mut Vec<u8>>

pub fn as_fn_def_mut(&mut self) -> Option<&mut FnPtr>

pub fn as_fn_ptr_mut(&mut self) -> Option<&mut FnPtr>

pub fn as_ptr_no_provenance_mut(&mut self) -> Option<&mut u128>

pub fn as_type_id_mut(&mut self) -> Option<&mut Ty>

pub fn as_raw_memory_mut(&mut self) -> Option<&mut Vec<Byte>>

pub fn as_var_mut(&mut self) -> Option<&mut ConstGenericDbVar>

pub fn as_global_mut(&mut self) -> Option<&mut GlobalDeclRef>

pub fn as_call_mut(&mut self) -> Option<(&mut FnPtr, &mut Vec<ConstantExpr>)>

pub fn as_trait_const_mut( &mut self, ) -> Option<(&mut TraitRef, &mut AssocConstId)>

pub fn as_vtable_ref_mut(&mut self) -> Option<&mut TraitRef>

pub fn as_discriminant_mut( &mut self, ) -> Option<(&mut TypeDeclRef, &mut VariantId)>

pub fn as_size_of_mut(&mut self) -> Option<&mut Ty>

pub fn as_align_of_mut(&mut self) -> Option<&mut Ty>

pub fn as_offset_of_mut( &mut self, ) -> Option<(&mut TypeDeclRef, &mut Option<VariantId>, &mut FieldId)>

pub fn as_opaque_mut(&mut self) -> Option<&mut String>

Source

impl ConstantExprKind

pub fn from_bits(scalar_ty: &ScalarTy, bits: u128) -> Option<Self>