pub enum IntegerValue {
Unsigned(UIntTy, u128),
Signed(IntTy, i128),
}Expand description
A scalar value.
Variants
Implementations
Sourceimpl IntegerValue
impl IntegerValue
Sourceimpl IntegerValue
impl IntegerValue
Sourceimpl IntegerValue
impl IntegerValue
pub fn as_unsigned_mut(&mut self) -> Option<(&mut UIntTy, &mut u128)>
pub fn as_signed_mut(&mut self) -> Option<(&mut IntTy, &mut i128)>
Sourceimpl IntegerValue
impl IntegerValue
pub fn variant_name(&self) -> &'static str
Sourceimpl IntegerValue
impl IntegerValue
pub fn variant_index_arity(&self) -> (u32, usize)
Sourceimpl IntegerValue
impl IntegerValue
fn ptr_size_max(ptr_size: ByteCount, signed: bool) -> u128
fn ptr_size_min(ptr_size: ByteCount, signed: bool) -> i128
pub fn ty(&self) -> IntegerTy
pub fn is_int(&self) -> bool
pub fn is_uint(&self) -> bool
Sourcepub fn as_uint(&self) -> Option<u128>
pub fn as_uint(&self) -> Option<u128>
When computing the result of binary operations, we convert the values to u128 then back to the target type (while performing dynamic checks of course).
pub fn uint_is_in_bounds(ptr_size: ByteCount, ty: UIntTy, v: u128) -> bool
pub fn from_unchecked_uint(ty: UIntTy, v: u128) -> IntegerValue
pub fn from_uint(ptr_size: ByteCount, ty: UIntTy, v: u128) -> Option<Self>
pub fn mk_usize(value: u128) -> Self
Sourcepub fn as_int(&self) -> Option<i128>
pub fn as_int(&self) -> Option<i128>
When computing the result of binary operations, we convert the values to i128 then back to the target type (while performing dynamic checks of course).
pub fn int_is_in_bounds(ptr_size: ByteCount, ty: IntTy, v: i128) -> bool
pub fn from_unchecked_int(ty: IntTy, v: i128) -> IntegerValue
Sourcepub fn to_bits(&self) -> u128
pub fn to_bits(&self) -> u128
Most integers are represented as u128 by rustc. We must be careful not to sign-extend.
Sourcepub fn from_le_bytes(ty: IntegerTy, bytes: [u8; 16]) -> Self
pub fn from_le_bytes(ty: IntegerTy, bytes: [u8; 16]) -> Self
Translates little endian bytes into a corresponding IntegerValue.
This needs to do the round-trip to the correct integer type to guarantee
that the values are correctly sign-extended (e.g. if the bytes encode -1i8, taking all 16 bytes
would lead to the value 255i128 instead of -1i128).
pub fn from_bits(ty: IntegerTy, bits: u128) -> Self
Sourcepub fn from_int(ptr_size: ByteCount, ty: IntTy, v: i128) -> Option<IntegerValue>
pub fn from_int(ptr_size: ByteCount, ty: IntTy, v: i128) -> Option<IntegerValue>
Warning: most constants are stored as u128 by rustc. When converting
to i128, it is not correct to do v as i128, we must reinterpret the
bits (see IntegerValue::from_le_bytes).
Sourcepub fn add(self, n: u128) -> Option<Self>
pub fn add(self, n: u128) -> Option<Self>
Increment the value, staying within the same integer type. Returns None on overflow.
pub fn to_constant(self) -> ConstantExpr