FloatValue

Struct FloatValue 

pub struct FloatValue {
    pub value: String,
    pub ty: FloatTy,
}
This is simlar to the Scalar value above. However, instead of storing the float value itself, we store its String representation. This allows to derive the Eq and Ord traits, which are not implemented for floats

Fields

value: Stringty: FloatTy